#!/usr/bin/env python

"""
usage: flask-rest-client.py [-h] [-a ARGS] [-v] command

A Python client for flask-restul.

positional arguments:
  command               A command, one of: getlist, getone, add, update,
                        delete

optional arguments:
  -h, --help            show this help message and exit
  -a ARGS, --args ARGS  Arguments: (1) task name or (2) URL-encoded form
                        parameters
  -v, --verbose         Print messages during actions.

additional arguments:
  getone      --args="<task_name>"
  add         --args="task_name=<task_name>&description=<description>"
  update      --args="task_name=<task_name>&description=<description>"
  delete      --args="<task_name>"
"""


from __future__ import print_function
import sys
import argparse
import requests


Base_URL = 'http://localhost:8001/tasks'


def dbg_msg(options, msg):
    """Print a message if verbose is on."""
    if options.verbose:
        print(msg)


def convertparams(params):
    """Convert parameters from string to dictionary."""
    paramdict = {}
    params = params.split('&')
    for param in params:
        key, value = param.split('=')
        paramdict[key] = value
    return paramdict


def getlist(options):
    """Get a list of all tasks."""
    url = Base_URL + '/gettask'
    headers = {'Accept': 'application/json'}
    response = requests.get(url, headers=headers)
    content = response.text
    json_content = response.json()
    print('content: {}'.format(content))
    dbg_msg(options, 'json_content: {}'.format(json_content))


def getone(options):
    """Get one task."""
    task_name = options.args
    url = Base_URL + '/gettask/' + task_name
    headers = {'Accept': 'application/json'}
    response = requests.get(url, headers=headers)
    content = response.text
    json_content = response.json()
    print('content: {}'.format(content))
    dbg_msg(options, 'json_content: {}'.format(json_content))
    dbg_msg(options, 'action:      {}'.format(json_content['action']))
    dbg_msg(options, 'task_name:   {}'.format(json_content['task_name']))
    dbg_msg(options, 'description: {}'.format(json_content['description']))


def addtask(options):
    """Add a task given its task_name and description."""
    params = convertparams(options.args)
    url = Base_URL + '/addtask'
    headers = {'Accept': 'application/json'}
    response = requests.post(url, headers=headers, data=params)
    content = response.text
    json_content = response.json()
    print('content: {}'.format(content))
    dbg_msg(options, 'action:      {}'.format(json_content['action']))


def updatetask(options):
    """Update a task given its task_name and a new description."""
    params = convertparams(options.args)
    url = Base_URL + '/updatetask'
    headers = {'Accept': 'application/json'}
    response = requests.put(url, headers=headers, data=params)
    content = response.text
    json_content = response.json()
    print('content: {}'.format(content))
    dbg_msg(options, 'json_content: {}'.format(json_content))
    dbg_msg(options, 'action:      {}'.format(json_content['action']))


def deletetask(options):
    """Delete a task."""
    task_name = options.args
    url = Base_URL + '/gettask/' + task_name
    headers = {'Accept': 'application/json'}
    response = requests.delete(url, headers=headers)
    content = response.text
    json_content = response.json()
    print('content: {}'.format(content))
    dbg_msg(options, 'json_content: {}'.format(json_content))
    dbg_msg(options, 'action:      {}'.format(json_content['action']))
    dbg_msg(options, 'task_name:   {}'.format(json_content['task_name']))
    dbg_msg(options, 'description: {}'.format(json_content['description']))


#
# Map command (string) to corresponding function.
Function_map = {
    'getlist': getlist,
    'getone': getone,
    'add': addtask,
    'update': updatetask,
    'delete': deletetask,
}


def main():
    description = """\
synopsis:
  a Python client for flask-restul.
"""
    epilog = """\
additional arguments:
  getone      --args="<task_name>"
  add         --args="task_name=<task_name>&description=<description>"
  update      --args="task_name=<task_name>&description=<description>"
  delete      --args="<task_name>"

examples:
  python flaskrest_client01.py getlist
  python flaskrest_client01.py getone --args="t03"
  python flaskrest_client01.py add --args="task_name=t11&description=task 11"
  python flaskrest_client01.py update \
--args="task_name=t11&description=task 11a"
  python flaskrest_client01.py delete --args="t03"
"""
    parser = argparse.ArgumentParser(
        description=description,
        epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "command",
        help="A command, one of: getlist, getone, add, update, delete"
    )
    parser.add_argument(
        "-a", "--args",
        help="Arguments: (1) task name or (2) form parameters"
    )
    parser.add_argument(
        "-v", "--verbose",
        action="store_true",
        help="Print messages during actions.",
    )
    options = parser.parse_args()
    fn = Function_map.get(options.command)
    if fn is not None:
        fn(options)
    else:
        sys.exit(__doc__)


if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()
